package com.usky.meeting.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.meeting.domain.MeetingFloor;
import com.usky.meeting.domain.MeetingRoom;
import com.usky.meeting.mapper.MeetingRoomMapper;
import com.usky.meeting.repository.MeetingRoomRepository;
import com.usky.meeting.service.MeetingFloorService;
import com.usky.meeting.service.MeetingInfoService;
import com.usky.meeting.service.MeetingRoomService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.meeting.service.vo.MeetingRoomRequestVO;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 会议室表 服务实现类
 * </p>
 *
 * @author zyj
 * @since 2024-03-08
 */
@Service
@RequiredArgsConstructor
public class MeetingRoomServiceImpl extends AbstractCrudService<MeetingRoomMapper, MeetingRoom> implements MeetingRoomService {

    @Autowired
    private MeetingRoomRepository dmMeetingRoomRepository;
    @Autowired
    private MeetingInfoService dmMeetingService;
    @Autowired
    private MeetingFloorService dmFloorService;

    @Override
    public Map<String,Object> FirstPageStatistic(String year){
        Map<String,Object> map = new HashMap<>();
        Map<String,Object> roomSummary = dmMeetingRoomRepository.roomSummary();
        map.put("roomSummary",roomSummary);
        Map<String,Object> meetingSummary = dmMeetingService.meetingSummary();
        map.put("meetingSummary",meetingSummary);
        Map<String,Object> meetingSummaryByYear = dmMeetingService.meetingSummaryByYear(year);
        map.put("meetingSummaryByYear",meetingSummaryByYear);

        return map;
    }

    @Override
    public CommonPage<MeetingRoom> dmMeetingRoomList(MeetingRoomRequestVO requestVO){
        IPage<MeetingRoom> page = new Page<>(requestVO.getCurrent(),requestVO.getSize());

        String floorId = null ;
        if(StringUtils.isNotBlank(requestVO.getFloorName())){
            QueryWrapper<MeetingFloor> query = Wrappers.query();
            query.select("floor_id").eq("floor_name",requestVO.getFloorName());
            List<Map<String,Object>> map = dmFloorService.listMaps(query);
            if(CollectionUtils.isNotEmpty(map)){
                floorId = map.get(0).get("floor_id").toString();
            }
        }

        LambdaQueryWrapper<MeetingRoom> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(StringUtils.isNotBlank(requestVO.getRoomName()),MeetingRoom::getRoomName,requestVO.getRoomName())
                .eq(StringUtils.isNotBlank(floorId),MeetingRoom::getFloorId,floorId)
                .eq(StringUtils.isNotBlank(requestVO.getStatus()),MeetingRoom::getStatus,requestVO.getStatus())
                .eq(null != requestVO.getRoomStatus(),MeetingRoom::getRoomStatus,requestVO.getRoomStatus())
                .orderByDesc(MeetingRoom::getRoomId);
        page = this.page(page,queryWrapper);
        if(CollectionUtils.isNotEmpty(page.getRecords())){
            //查询楼层信息和本地存储信息
            List<BigInteger> floorIdList = new ArrayList<>();
            List<BigInteger> storageIdList = new ArrayList<>();
            for (int i = 0; i < page.getRecords().size(); i++) {
                BigInteger floorId1 = new BigInteger(page.getRecords().get(i).getFloorId());
                BigInteger storageId = new BigInteger(page.getRecords().get(i).getStorageId());
                if(!floorIdList.contains(floorId1)){
                    floorIdList.add(floorId1);
                }
                if(!storageIdList.contains(storageId)){
                    storageIdList.add(storageId);
                }
            }
            LambdaQueryWrapper<MeetingFloor> floorQuery = Wrappers.lambdaQuery();
            floorQuery.in(CollectionUtils.isNotEmpty(floorIdList), MeetingFloor::getFloorId,floorIdList);
            List<MeetingFloor> floorList = dmFloorService.list(floorQuery);

            //会议室匹配对应楼层信息和本地存储信息
            for (int i = 0; i < page.getRecords().size(); i++) {
                if(CollectionUtils.isNotEmpty(floorList)){
                    Long frId = Long.valueOf(page.getRecords().get(i).getFloorId());
                    for (int j = 0; j < floorList.size(); j++) {
                        if(frId.equals(floorList.get(j).getFloorId())){
                            page.getRecords().get(i).setMeetingFloor(floorList.get(j));
                            break;
                        }
                    }
                }
            }
        }


        return new CommonPage<>(page.getRecords(),page.getTotal(),requestVO.getSize(),requestVO.getCurrent());
    }
}
